#//-----------------------------------------------
#//TUSB-KINSxx用Pythonサンプルプログラム
#// 処理関数等
#//
#//2024_04_10　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import tkinter as tk
import threading
from tkinter import messagebox
import time

def ShowInfo(str):
    """文字列を表示する"""
    messagebox.showinfo("",str)

def GetLabel(master,text,relx,rely):
    """ラベルを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    表示する文字列
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成されたラベルインスタンス
    """
    label = tk.Label(master,text=text)
    label.place(relx=relx,rely=rely,anchor=tk.W)
    return label

def GetChkBox(master,text,relx,rely,val):
    """チェックボックスを生成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    表示する文字列
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    val         ----    variableを対応させるwidget変数(例 BooleanVar())

    returns:
    Checkbutton ----    生成されたチェックボックスのインスタンス
    """
    chkbox = tk.Checkbutton(master,text=text,variable=val)
    chkbox.place(relx=relx,rely=rely,anchor=tk.W)
    return chkbox

def GetLabelDisp(master,text,relw,relx,rely):
    """表示用ラベルを生成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    生成時に表示する文字列
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成された表示用ラベルのインスタンス
    """
    label = tk.Label(master,text=text,relief="groove")
    label.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    return label

def IsBitSet(data,bit):
    if ((data>>bit) & 1) == 1:return True
    else:return False

def GenList(N,initval):
    """list generator with value initialize"""
    ret = list()
    for i in range(N):
        ret.append(initval)
    return ret

def GenList2D(N1,N2,initval):
    """2D list array generator with value initialize"""
    ret = list()
    for i in range(N1):
        ret.append(GenList(N2,initval))
    return ret

def GenTkVarList(N,cls,initval):
    """Tkinter Var list generator with value initialize
    
    arguments:
    N           ----    list length
    cls         ----    tkinter variable class
    initval     ----    initial value
    
    returns:
    list        ----    generated list
    """
    ret = list()
    for i in range(N):
        ret.append(cls(value=initval))
    return ret

def GenTkVarList2D(N1,N2,cls,initval):
    """Tkinter Var 2D list generator with value initialize
    
    arguments:
    N1          ----    first list index length
    N2          ----    second list index length
    cls         ----    tkinter variable class
    initval     ----    initial value
    
    returns:
    list        ----    generated list
    """
    r = list()
    for i in range(N1):
        r.append(GenTkVarList(N2,cls,initval))
    return r

# タイマースレッド管理クラス ////////////////////////////////////
#厳密なインターバルではない。関数処理が終わったら指定時間待つだけ。
#from builtins import function
class intTT():
    """タイマースレッド管理クラス
    厳密なインターバル処理ではない。
    関数処理が終わったら指定時間待って再度実行しつづるループ。
    """
    thread :threading.Thread = None # スレッドクラスオブジェクト
    thread_en : bool = None         # スレッド実行イネーブルフラグ
    call_func = None                # ループで実行する関数
    interval = None                 # ループの待ち時間

    def __init__(self,inttime,callf):
        """初期化"""
        self.interval = inttime # インターバルタイム
        self.call_func = callf  # 処理関数登録
    
    def ThreadF(self):
        """スレッド関数(ループ)"""
        while self.thread_en:
            self.call_func()
            time.sleep(self.interval)

    def Start(self):
        """スレッド開始処理"""
        if self.thread == None:
            self.thread_en = True
            self.thread = threading.Thread(target=self.ThreadF)
            self.thread.start()
    
    def Stop(self):
        """スレッド停止処理"""
        if self.thread_en != None:
            self.thread_en = False
            self.thread.join(self.interval+0.1)
            self.thread = None 

