#//-----------------------------------------------
#//TUSB-KINSxx用サンプルプログラム
#//Python 用モジュール
#//
#//2024_04_10　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------
import TUSBKINS
import sys

from SpFuncs import *
from functools import partial

class Tusbkins_Device:
    """TUSB-KINSxx Device Control"""

    DevId = None
    mon_t = None
    lock = threading.Lock() # デバイスコマンドが同時実行されないために使用するロック

    def __init__(self,DevId):
        """constructor
                
        arguments:
        DevId           ----    Device id 0~15
        """
        ret = TUSBKINS.TUSBKINS_Device_Open(DevId)
        if ret == 0:
            self.DevId = DevId
            self.form.protocol("WM_DELETE_WINDOW", self.DevClose)   # フォームが閉じられる時に処理を行うため
            if self.DevInit():
                self.mon_t = intTT(0.5,self.Monitor)    # タイマースレッドを開始してタイマーからMonitorメソッドを呼び出させる
                self.mon_t.Start()
                return
            TUSBKINS.TUSBKINS_Device_Close(DevId)
        ShowInfo(TUSBKINS.GetErrMessage(ret))
        self.form.destroy()                             # 何か問題が有ったら終了とする

    def DevInit(self):
        """デバイスの状態を確認して画面表示に反映する"""
        # デバイスのタイプを確認する
        ret = TUSBKINS.TUSBKINS_Type(self.DevId)
        if ret[0] != 0:
            ShowInfo(TUSBKINS.GetErrMessage(ret[0]))
            return False
        if ret[1] == 0: self.form.title('ID = '+str(self.DevId)+" TUSB-INS4/4")
        if ret[1] == 1: self.form.title('ID = '+str(self.DevId)+" TUSB-INS08O")
        if ret[1] == 2: self.form.title('ID = '+str(self.DevId)+" TUSB-INS08I")
        if ret[1] == 3: self.form.title('ID = '+str(self.DevId)+" TUSB-INS8/8")
        if ret[1] == 4: self.form.title('ID = '+str(self.DevId)+" TUSB-INS16O")
        if ret[1] == 5: self.form.title('ID = '+str(self.DevId)+" TUSB-INS16I")
        # 出力ビット状態High Lowを読み出してチェックボックスに反映する
        ret = TUSBKINS.TUSBKINS_Read(self.DevId)
        if ret[0] != 0:
            ShowInfo(TUSBKINS.GetErrMessage(ret[0]))
            return False
        for bit in range(16):
            self.ChkBit[bit].set(IsBitSet(ret[2],bit))
        # ラジオボタンとチェックボックス変更時のコマンドハンドラを設定する
        for bit in range(16):
            self.chkb[bit].config(command=partial(self.Check_on))
        return True
    
    def DevClose(self):
        """デバイスを閉じる時の処理"""
        if self.DevId != None:
            self.mon_t.Stop()
            TUSBKINS.TUSBKINS_Device_Close(self.DevId)
            self.form.destroy()

    def Check_on(self):
        """出力ビットが変更された時のハンドラ"""
        data = 0
        for bit in range(16):
            data += (1<<bit) if self.ChkBit[bit].get() else 0
        if self.lock.acquire(timeout=1):
            ret = TUSBKINS.TUSBKINS_Write(self.DevId,data)
            self.lock.release()
            if ret != 0:
                ShowInfo(TUSBKINS.GetErrMessage(ret))

    # timer -------------------------------------
    def Monitor(self):
        """入力ビット確認処理 タイマーから呼び出される"""
        if self.DevId != None:
            if self.lock.acquire(timeout=1):
                ret = TUSBKINS.TUSBKINS_Read(self.DevId)
                self.lock.release()
                if ret[0] == 0:
                    for bit in range(16):
                        if IsBitSet(ret[1],bit): self.StatusDisp[bit]['text'] = "On"
                        else: self.StatusDisp[bit]['text'] = "Off"

    # form generation ---------------------------
    form = tk.Tk()
    form.geometry('160x332')
    form.title('TUSB-KINSxx Sample Program')
    form.attributes("-toolwindow",True)

    chkb = GenList(16,None)                      # チェックボックス
    StatusDisp = GenList(16,None)                # 入力状態表示ラベル
    ChkBit = GenTkVarList(16,tk.BooleanVar,False)# チェックボックスのチェック状態

    # 表示ラベル
    GetLabel(form,"Output",0.15,0.04)
    GetLabel(form,"Input",0.6,0.04)
    # チェックボックスと入力表示
    for bit in range(16):
        chkb[bit] = GetChkBox(form,str(bit),0.18,0.11+0.053*bit,ChkBit[bit])
        StatusDisp[bit] = GetLabelDisp(form,"",0.3,0.55,0.11+0.053*bit)

# テスト用 ---------------------------------
if __name__=="__main__":
    DevId = 0
    if( len(sys.argv) >= 2 ):
        if sys.argv[1].isdigit():
            DevId = int(sys.argv[1])    # コマンドライン引数でID設定
    tmp = Tusbkins_Device(DevId=DevId)
    tmp.form.mainloop()
